package com.novaordis.thread194629;

import org.apache.log4j.Logger;
import org.infinispan.Cache;
import org.infinispan.manager.CacheManager;
import org.infinispan.tree.Fqn;
import org.infinispan.tree.Node;
import org.infinispan.tree.TreeCache;
//import org.slf4j.Logger;
//import org.slf4j.LoggerFactory;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.List;
import java.util.concurrent.Callable;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Future;

public class RCBJTest1 implements Serializable
{
    private static final long   serialVersionUID                 = -7108137415096982449L;

    private static final String SOURCE_CACHE_SIGNATURE_NAME      = "Source";
    private static final String DESTINATION_CACHE_SIGNATURE_NAME = "Destination";

    private static final String SOURCE_CHILD_NODE_PREFIX         = "ChildNode";
    private static final int    NUM_SOURCE_NODES                 = 50;

    private static final Logger log                              = Logger.getLogger(TestMultiWriter.class);

//  private final String        dataSourceCacheManagerName ;

    public RCBJTest1()
    {

    }

    protected CacheManager getCacheManager()
    {
        return TestHelper.getInstance().getCacheManager();
    }

    public static void main(String [] args) throws Exception
    {
        new RCBJTest1().runTest();
    }

    public void runTest() throws Exception
    {
        log.info("runTest starting up");

        TestHelper.getInstance();

        final ITreeCacheSignatureTest sourceSignature =
            new MWTestCacheSignature1(SOURCE_CACHE_SIGNATURE_NAME);
        final Cache<Object, Object> sourceCache = TestHelper.getInstance().
            createIfAbsent(sourceSignature);

        final ITreeCacheSignatureTest destinationSignature =
            new MWTestCacheSignature1(DESTINATION_CACHE_SIGNATURE_NAME);
        final Cache<Object, Object> destinationCache = TestHelper.getInstance().
            createIfAbsent(destinationSignature);

        final List<Fqn> sourceChildPaths = populateSourceCache(sourceCache);

        log.info("list size: " + sourceChildPaths.size());

        runCallable(sourceCache, destinationCache, sourceChildPaths);

        log.info("runTest complete");
    }

    private List<Fqn> populateSourceCache(
        final Cache<Object, Object> sourceCache)
    {
        final List<Fqn> childPaths = new ArrayList<Fqn>(
            NUM_SOURCE_NODES);

        final TreeCache<Object, Object> treeCache = TestHelper.getInstance().getTreeCache(sourceCache);
        final Node<Object, Object> rootNode = treeCache.getRoot();
        int i = 0;
        for (i = 0; i < NUM_SOURCE_NODES; i++)
        {
            final String childNodeName = SOURCE_CHILD_NODE_PREFIX
                                         + Integer.toString(i);
            final Fqn childPath = Fqn.fromString(childNodeName);
            rootNode.addChild(childPath);
            childPaths.add(childPath);
        }
        System.out.println("i=" + i);
        return childPaths;
    }

    private void runCallable(final Cache<Object, Object> sourceCache,
                             final Cache<Object, Object> destinationCache,
                             final List<Fqn> sourceChildPaths) throws Exception
    {
        final String sourceCacheName = sourceCache.getName();
        final String destinationCacheName = destinationCache.getName();

        log.info("list size: " + sourceChildPaths.size());

        Object[] keys = sourceChildPaths.toArray();
        ArrayList<Fqn> pathsToProcess = new ArrayList<Fqn>(sourceChildPaths);

        long numPathsProcessed = 0;

        log.info("List Size: " + pathsToProcess.size());

        if (pathsToProcess.size() > 0)
        {
            final ExecutorService pool = TestHelper.getInstance().getExecutorService();
            final List<Future<Long>> futureResults = new ArrayList<Future<Long>>();
            int count = 0;
            for (final Fqn pathToProcess : pathsToProcess)
            {
                final Callable<Long> callable =
                    new MWTestCallable1(this, sourceCacheName,
                                        destinationCacheName, pathToProcess);

                final Future<Long> futureResult = pool.submit(callable);
                futureResults.add(futureResult);
                count++;
            }

            Thread.sleep(20000);

            log.info("iterated: " + count);

            for (final Future<Long> futureResult : futureResults)
            {
                // this will throw an exception if the key was not found in the destination cache
                numPathsProcessed += futureResult.get().longValue();
            }
        }
    }
}

class MWTestCallable1 implements Callable<Long>, Serializable
{
    private static final long     serialVersionUID = -1572063052116806145L;
    private static final Logger   log              = Logger.getLogger(TestMultiWriter.class);

    private final RCBJTest1 mwTest;
    private final String          sourceCacheName;
    private final String          destinationCacheName;
    private final Fqn   pathToProcess;

    protected MWTestCallable1(final RCBJTest1 mwTest,
                              final String sourceCacheName, final String destinationCacheName,
                              final Fqn pathToProcess)
    {
        this.mwTest = mwTest;
        this.sourceCacheName = sourceCacheName;
        this.destinationCacheName = destinationCacheName;
        this.pathToProcess = pathToProcess;
    }

    @Override
    public Long call() throws Exception
    {
        final CacheManager cacheManager = TestHelper.getInstance().getCacheManager();

        final Cache<Object, Object> sourceCache = cacheManager
            .getCache(sourceCacheName);
        final Cache<Object, Object> destinationCache = cacheManager
            .getCache(destinationCacheName);

        final TreeCache<Object, Object> sourceTreeCache = TestHelper.getInstance().getTreeCache(sourceCache);

        TreeCache<Object, Object> destinationTreeCache = TestHelper.getInstance().getTreeCache(destinationCache);


        final Node<Object, Object> sourceTreeNode = sourceTreeCache.getNode(pathToProcess);
        final Node<Object, Object> destinationTreeNode = destinationTreeCache
            .getRoot();
        System.out.println("pathToProcess=" + pathToProcess);
        String valueToPutOnRootNode = sourceTreeNode.getFqn().toString();

        if (!valueToPutOnRootNode.equals(pathToProcess.toString()))
        {
            throw new Exception("The node names do not match: "
                                + valueToPutOnRootNode + ", " + pathToProcess.toString());
        }

        if(valueToPutOnRootNode.startsWith("/"))
        {
            valueToPutOnRootNode = valueToPutOnRootNode.substring(1);
        }

        destinationTreeNode.put(valueToPutOnRootNode, valueToPutOnRootNode);
        log.info("Put key and value '" + valueToPutOnRootNode + "' on cache " + destinationCacheName + " root node");

        //Can we read it back out?
        final String key = pathToProcess.toString().substring(1);
        Object value = destinationTreeNode.get(key);

        if (value == null)
        {
            // test fails
            throw new Exception("could not read " + key + " from " + destinationCache.getName());
        }

        return Long.valueOf(1);
    }
}

class MWTestCacheSignature1 implements ITreeCacheSignatureTest
{
    private static final long serialVersionUID = 6982205318901622122L;

    private final String      name;

    protected MWTestCacheSignature1(final String name)
    {
        this.name = name;
    }

    protected String getName()
    {
        return name;
    }

    public boolean equals(final Object o)
    {
        if (o == this)
        {
            return true;
        }

        if (!(o instanceof MWTestCacheSignature1))
        {
            return false;
        }

        final MWTestCacheSignature1 sig = (MWTestCacheSignature1) o;
        return name.equals(sig.getName());
    }

}